@@ -1,16 +1,14 @@
 use std::net::SocketAddr;
 
-use super::Transport;
-use crate::{
-    config::{NoiseConfig, TransportConfig},
-    helper::set_tcp_keepalive,
-};
+use super::{SocketOpts, TcpTransport, Transport};
+use crate::config::{NoiseConfig, TransportConfig};
 use anyhow::{anyhow, Context, Result};
 use async_trait::async_trait;
 use snowstorm::{Builder, NoiseParams, NoiseStream};
 use tokio::net::{TcpListener, TcpStream, ToSocketAddrs};
 
 pub struct NoiseTransport {
+    tcp: TcpTransport,
     config: NoiseConfig,
     params: NoiseParams,
     local_private_key: Vec<u8>,
@@ -36,9 +34,12 @@ impl NoiseTransport {
 #[async_trait]
 impl Transport for NoiseTransport {
     type Acceptor = TcpListener;
+    type RawStream = TcpStream;
     type Stream = snowstorm::stream::NoiseStream<TcpStream>;
 
-    async fn new(config: &TransportConfig) -> Result<Self> {
+    fn new(config: &TransportConfig) -> Result<Self> {
+        let tcp = TcpTransport::new(config)?;
+
         let config = match &config.noise {
             Some(v) => v.clone(),
             None => return Err(anyhow!("Missing noise config")),
@@ -60,6 +61,7 @@ impl Transport for NoiseTransport {
         let params: NoiseParams = config.pattern.parse()?;
 
         Ok(NoiseTransport {
+            tcp,
             config,
             params,
             local_private_key,
@@ -67,23 +69,38 @@ impl Transport for NoiseTransport {
         })
     }
 
+    fn hint(conn: &Self::Stream, opt: SocketOpts) {
+        opt.apply(conn.get_inner());
+    }
+
     async fn bind<T: ToSocketAddrs + Send + Sync>(&self, addr: T) -> Result<Self::Acceptor> {
         Ok(TcpListener::bind(addr).await?)
     }
 
-    async fn accept(&self, a: &Self::Acceptor) -> Result<(Self::Stream, SocketAddr)> {
-        let (conn, addr) = a.accept().await?;
-        set_tcp_keepalive(&conn);
+    async fn accept(&self, a: &Self::Acceptor) -> Result<(Self::RawStream, SocketAddr)> {
+        self.tcp
+            .accept(a)
+            .await
+            .with_context(|| "Failed to accept TCP connection")
+    }
 
-        let conn = NoiseStream::handshake(conn, self.builder().build_responder()?).await?;
-        Ok((conn, addr))
+    async fn handshake(&self, conn: Self::RawStream) -> Result<Self::Stream> {
+        let conn = NoiseStream::handshake(conn, self.builder().build_responder()?)
+            .await
+            .with_context(|| "Failed to do noise handshake")?;
+        Ok(conn)
     }
 
     async fn connect(&self, addr: &str) -> Result<Self::Stream> {
-        let conn = TcpStream::connect(addr).await?;
-        set_tcp_keepalive(&conn);
+        let conn = self
+            .tcp
+            .connect(addr)
+            .await
+            .with_context(|| "Failed to connect TCP socket")?;
 
-        let conn = NoiseStream::handshake(conn, self.builder().build_initiator()?).await?;
+        let conn = NoiseStream::handshake(conn, self.builder().build_initiator()?)
+            .await
+            .with_context(|| "Failed to do noise handshake")?;
         return Ok(conn);
     }
 }
